/* BdfServer_JsonProducers - Copyright (c) 2022-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.edition;

import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.EditionDomain;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.fiche.FicheBlocks;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.logging.ErrorMessageException;


/**
 *
 * @author Vincent Calame
 */
public final class EditionJsonProducerFactory {

    private EditionJsonProducerFactory() {

    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws ErrorMessageException {
        String json = parameters.getOutput();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        MessageLocalisation messageLocalisation = parameters.getMessageLocalisation();
        ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, messageLocalisation);
        switch (json) {
            case EditionDomain.SECTION_JSON: {
                boolean done = false;
                if (bdfCommandResult != null) {
                    FicheBlocks ficheBlocks = (FicheBlocks) bdfCommandResult.getResultObject(BdfInstructionConstants.FICHEBLOCKS_OBJ);
                    Corpus corpus = (Corpus) bdfCommandResult.getResultObject(BdfInstructionConstants.CORPUS_OBJ);
                    if ((ficheBlocks != null) && (corpus != null)) {
                        jsonProducer.add(new SectionHtmlJsonProperty(parameters, corpus, ficheBlocks));
                        done = true;
                    }
                }
                if (!done) {
                    throw BdfErrors.missingCommandResultJson(EditionDomain.SECTION_JSON);
                }
                break;
            }
        }
        if (jsonProducer.isEmpty()) {
            return null;
        } else {
            return jsonProducer;
        }
    }

}
